/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QDebug>
#include <QWindow>

#include <wayland-client.h>

#include "DesQLayerShell.hpp"
#include "DesQLayerShellImpl.hpp"
#include "DesQWlGlobal.hpp"

#include "qwayland-wlr-layer-shell-unstable-v1.h"

// Easier to type
typedef struct ::zwlr_layer_shell_v1 wlrLayerShell;
typedef struct ::zwlr_layer_surface_v1 wlrDesQLayerSurface;

/*
	*
	* DesQLayerShell Private
	*
*/
DesQLayerShellPrivate::DesQLayerShellPrivate( wlrLayerShell *lShell, DesQLayerShell *ptr ) : QtWayland::zwlr_layer_shell_v1( lShell ) {

	q_ptr = ptr;
	shellPtr = lShell;
}

zwlr_layer_surface_v1 *DesQLayerShellPrivate::getDesQLayerSurface( QWindow *window, DesQLayerShell::LayerType lt, const QString &lyrName ) {

	if ( !window ) {
		qDebug() << "No window found to create surface";
		return nullptr;
	}

	auto surface = DesQWayland::getWlSurface( window );
	if ( !surface ) {
		qDebug() << "No surface for the given window";
		return nullptr;
	}

	auto output = DesQWayland::getWlOutput( window->screen() );
	if ( !output ) {
		qDebug() << "No output for the given window";
		return nullptr;
	}

	return get_layer_surface( surface, output, static_cast<uint32_t>( lt ), lyrName );
}

/*
	*
	* DesQLayerShell
	*
*/

DesQLayerShell::DesQLayerShell( wlrLayerShell *lShell, QObject *parent ) : QObject( parent ) {

	d_ptr = new DesQLayerShellPrivate( lShell, this );
}

wlrDesQLayerSurface* DesQLayerShell::getDesQLayerSurface( QWindow *window, LayerType layer, const QString &lnamespace ) {

	Q_D( DesQLayerShell );
	return d->getDesQLayerSurface( window, layer, lnamespace );
}

/*
	*
	* DesQLayerSurface Private
	*
*/

DesQLayerSurfacePrivate::DesQLayerSurfacePrivate( QWindow *window, DesQLayerShell *lShell, DesQLayerShell::LayerType type, DesQLayerSurface *ptr ) :
		QtWayland::zwlr_layer_surface_v1( lShell->getDesQLayerSurface( window, type, "qt" ) ),
		q_ptr( ptr )
	{

	m_window = window;
	isConfigured = false;

	m_exclusiveZone = 0;
	m_keyboardInteractivity = 2;
	m_surfaceSize = QSize();
	m_margins = QMargins( 0, 0, 0, 0 );
};

void DesQLayerSurfacePrivate::setLayer( DesQLayerShell::LayerType type ) {

	set_layer( (uint32_t)type );
};

void DesQLayerSurfacePrivate::apply() {

	isConfigured = false;
	set_anchor( static_cast<uint32_t>( m_anchors ) );
	set_exclusive_zone( m_exclusiveZone );
	set_keyboard_interactivity( m_keyboardInteractivity );

	if ( m_surfaceSize.isValid() )
		set_size( m_surfaceSize.width(), m_surfaceSize.height() );

	else
		set_size( m_window->width(), m_window->height() );

	set_margin( m_margins.top(), m_margins.right(), m_margins.bottom(), m_margins.left() );

	wl_surface_commit( DesQWayland::getWlSurface( m_window ) );
	wl_display_roundtrip( DesQWayland::getWlDisplay() );
}

void DesQLayerSurfacePrivate::zwlr_layer_surface_v1_configure( uint32_t serial, uint32_t width, uint32_t height ) {

	Q_Q( DesQLayerSurface );

	isConfigured = true;
	m_surfacePendingSize = QSize( width, height );

	ack_configure( serial );
	m_window->resize( m_surfacePendingSize );

	emit q->configured( m_surfacePendingSize );
};

void DesQLayerSurfacePrivate::zwlr_layer_surface_v1_closed() {

	m_window->close();
};

/*
	*
	* DesQLayerSurface
	*
*/

DesQLayerSurface::DesQLayerSurface( QWindow *window, QObject *parent ) : QObject( parent ) {

	mWindow = window;
};

void DesQLayerSurface::setup( DesQLayerShell *lShell, DesQLayerShell::LayerType type ) {

	d_ptr = new DesQLayerSurfacePrivate( mWindow, lShell, type, this );
}

void DesQLayerSurface::apply() {

	Q_D( DesQLayerSurface );
	d->apply();
};

void DesQLayerSurface::setSurfaceSize( const QSize &surfaceSize ) {

	Q_D( DesQLayerSurface );
	d->m_surfaceSize = surfaceSize;
};

void DesQLayerSurface::setAnchors( const SurfaceAnchors &anchors ) {

	Q_D( DesQLayerSurface );
	d->m_anchors = anchors;
};

void DesQLayerSurface::setExclusiveZone( int exclusiveZone ) {

	Q_D( DesQLayerSurface );
	d->m_exclusiveZone = exclusiveZone;
};

void DesQLayerSurface::setMargins( const QMargins &margins ) {

	Q_D( DesQLayerSurface );
	d->m_margins = margins;
};

void DesQLayerSurface::setKeyboardInteractivity( DesQLayerSurface::FocusType focusType ) {

	Q_D( DesQLayerSurface );
	d->m_keyboardInteractivity = ( uint )focusType;
};

void DesQLayerSurface::setLayer( DesQLayerShell::LayerType type ) {

	Q_D( DesQLayerSurface );
	d->setLayer( type );
}
